import { Fragment as _Fragment, jsx as _jsx } from "react/jsx-runtime";
import { effect } from '@preact/signals-react';
import { useMemo, } from 'react';
import { useLifecycle } from '../hooks';
import { getElementInput, handleFieldEvent, initializeFieldStore, } from '../utils';
/**
 * Headless form field that provides reactive properties and state.
 */
export function Field({ children, type, ...props }) {
    // Destructure props
    const { of: form, name } = props;
    // Get store of specified field
    const field = useMemo(() => initializeFieldStore(form, name), [form, name]);
    // Use lifecycle of field
    useLifecycle({ ...props, store: field });
    return (_jsx(_Fragment, { children: children(useMemo(() => ({
            name,
            value: field.value,
            error: field.error,
            active: field.active,
            touched: field.touched,
            dirty: field.dirty,
        }), [field, name]), useMemo(() => ({
            name,
            ref(element) {
                if (element) {
                    // Add element to elements
                    field.elements.value = [...field.elements.value, element];
                    // Create effect that replaces initial input and input of field with
                    // initial input of element if both is "undefined", so that dirty
                    // state also resets to "false" when user removes input
                    effect(() => {
                        if (field.startValue.value === undefined &&
                            field.value.peek() === undefined) {
                            setTimeout(() => {
                                const input = getElementInput(element, field, type);
                                field.startValue.value = input;
                                field.value.value = input;
                            });
                        }
                    });
                }
            },
            onChange(event) {
                handleFieldEvent(form, field, name, event, ['touched', 'change'], getElementInput(event.currentTarget, field, type));
            },
            onBlur(event) {
                handleFieldEvent(form, field, name, event, ['touched', 'blur']);
            },
        }), [field, form, name, type])) }));
}
